/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSRulerMarker;

import dstep.appkit.NSEvent;
import dstep.appkit.NSImage;
import dstep.appkit.NSRulerView;
import dstep.applicationservices.coregraphics.CGBase;
import dstep.foundation.NSCoder;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObject;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSRulerMarker : NSObject, INSCopying, INSCoding
{
	mixin (ObjcWrap);
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}

	NSRulerMarker initWithRulerView (NSRulerView ruler, CGFloat location, NSImage image, NSPoint imageOrigin)
	{
		id result = invokeObjcSelf!(id, "initWithRulerView:markerLocation:image:imageOrigin:", NSRulerView, CGFloat, NSImage, NSPoint)(ruler, location, image, imageOrigin);
		return result is this.objcObject ? this : (result !is null ? new NSRulerMarker(result) : null);
	}

	this (NSRulerView ruler, CGFloat location, NSImage image, NSPoint imageOrigin)
	{
		super(NSRulerMarker.alloc.initWithRulerView(ruler, location, image, imageOrigin).objcObject);
	}

	NSRulerView ruler ()
	{
		return invokeObjcSelf!(NSRulerView, "ruler");
	}

	void setMarkerLocation (CGFloat location)
	{
		return invokeObjcSelf!(void, "setMarkerLocation:", CGFloat)(location);
	}

	CGFloat markerLocation ()
	{
		return invokeObjcSelf!(CGFloat, "markerLocation");
	}

	void setImage (NSImage image)
	{
		return invokeObjcSelf!(void, "setImage:", NSImage)(image);
	}

	NSImage image ()
	{
		return invokeObjcSelf!(NSImage, "image");
	}

	void setImageOrigin (NSPoint imageOrigin)
	{
		return invokeObjcSelf!(void, "setImageOrigin:", NSPoint)(imageOrigin);
	}

	NSPoint imageOrigin ()
	{
		return invokeObjcSelf!(NSPoint, "imageOrigin");
	}

	void setMovable (bool flag)
	{
		return invokeObjcSelf!(void, "setMovable:", bool)(flag);
	}

	void setRemovable (bool flag)
	{
		return invokeObjcSelf!(void, "setRemovable:", bool)(flag);
	}

	bool isMovable ()
	{
		return invokeObjcSelf!(bool, "isMovable");
	}

	bool isRemovable ()
	{
		return invokeObjcSelf!(bool, "isRemovable");
	}

	bool isDragging ()
	{
		return invokeObjcSelf!(bool, "isDragging");
	}

	void setRepresentedObject (INSCopying representedObject)
	{
		return invokeObjcSelf!(void, "setRepresentedObject:", INSCopying)(representedObject);
	}

	INSCopying representedObject ()
	{
		return invokeObjcSelf!(INSCopying, "representedObject");
	}

	NSRect imageRectInRuler ()
	{
		return invokeObjcSelf!(NSRect, "imageRectInRuler");
	}

	CGFloat thicknessRequiredInRuler ()
	{
		return invokeObjcSelf!(CGFloat, "thicknessRequiredInRuler");
	}

	void drawRect (NSRect rect)
	{
		return invokeObjcSelf!(void, "drawRect:", NSRect)(rect);
	}

	bool trackMouse (NSEvent mouseDownEvent, bool isAdding)
	{
		return invokeObjcSelf!(bool, "trackMouse:adding:", NSEvent, bool)(mouseDownEvent, isAdding);
	}

}

